﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System.ComponentModel;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.SRM;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  Agv交互系统
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("Agv交互系统")]
    public class AgvSystems : DeviceSystem<Station>
    {
        protected override bool ParallelDo => true;

        protected override bool SaveLogsToFile => true;

        private List<Station> devs = new List<Station>();
        private List<SRM> srms = new List<SRM>();

        public AgvSystems()
        {
            devs = Device.All.Where(v => v.HasFlag(DeviceFlags.入库, DeviceFlags.巷道口)).Select(v => new Station(v, this.World)).ToList();
            srms = Device.All.Where(v => v.HasProtocol(typeof(ISRM520))).Select(v => new SRM(v, this.World)).ToList();
        }

        public override void Do(Station obj)
        {
            if (obj.Entity.HasFlag(DeviceFlags.出库))
            {
                List<WCS_TaskInfo> taskInfos = new List<WCS_TaskInfo>();
                SqlSugarHelper.Do(db =>
                {
                    //获取所有未结束的叫料及背负式补空AGV任务
                    var agvTaks = db.Default.Queryable<WCS_AgvTaskInfo>().Where(v => (v.TaskType == AGVTaskType.CallMaterial || v.TaskType == AGVTaskType.CallForMaterial || v.TaskType == AGVTaskType.ForkliftFilling))
                                                                         .Where(v => v.Status < AGVTaskStatus.MissionCompleted)
                                                                         .SplitTable(tabs => tabs.Take(2)).ToList();
                    foreach (var agv in agvTaks)
                    {
                        //取货点安全交互
                        if (agv.AgvStatus == AGVTaskStatus.RequestOrPermission2 && agv.Status != AGVTaskStatus.Complete2)
                        {
                            agv.Status = AGVTaskStatus.Complete2;
                            db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                            AgvApi.ContinueTask(agv.AgvID, agv.Station);
                        }
                        //完成任务
                        else if (agv.AgvStatus == AGVTaskStatus.MissionCompleted && agv.Status != AGVTaskStatus.MissionCompleted)
                        {
                            if (agv.TaskType is AGVTaskType.CallForMaterial or AGVTaskType.ForkliftFilling or AGVTaskType.CallMaterial)
                            {
                                try
                                {
                                    var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.AgvTaskID == agv.ID);
                                    if (taskInfo == null)
                                    {
                                        World.Log($"未找到AGV任务{agv.ID}对应WCS任务");
                                        continue;
                                    }
                                    //更新AGV任务状态
                                    agv.Status = AGVTaskStatus.MissionCompleted;
                                    db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                                    //更新WCS任务状态
                                    taskInfo.Status = Entity.TaskStatus.Finish;
                                    taskInfo.EedTime = DateTime.Now;
                                    db.Default.Updateable(taskInfo).ExecuteCommand();
                                    taskInfo.AddWCS_TASK_DTL(db, "agv", "任务完成");
                                    taskInfo.CompleteOrCancelTasks(db);

                                    //通知WMS
                                    taskInfos.Add(taskInfo);
                                }
                                catch (Exception ex)
                                {
                                    World.Log(ex.Message);
                                    continue;
                                }
                            }
                            else
                            {
                                agv.Status = AGVTaskStatus.MissionCompleted;
                                db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                            }
                        }
                        else if (agv.AgvStatus == AGVTaskStatus.Cancel && agv.Status != AGVTaskStatus.Cancel)
                        {
                            //var taskInfo = db.Default.Queryable<WCS_TaskInfo>().First(v => v.AgvTaskID == agv.ID);
                            //if (taskInfo == null)
                            //{
                            //    World.Log($"未找到AGV任务{agv.ID}对应WCS任务");
                            //    continue;
                            //}
                            //try { WmsApi. } catch (Exception ex) { World.Log(ex.Message, LogLevelEnum.Mid); }

                            ////更新AGV任务状态
                            //agv.Status = AGVTaskStatus.Cancel;
                            //db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                            ////更新WCS任务状态
                            //taskInfo.Status = Entity.TaskStatus.Cancel;
                            //taskInfo.EedTime = DateTime.Now;
                            //db.Default.Updateable(taskInfo).ExecuteCommand();
                            //taskInfo.AddWCS_TASK_DTL(db, "agv", "任务完成");
                            //taskInfo.CompleteOrCancelTasks(db);
                        }
                    }
                });
                foreach (var item in taskInfos)
                {
                    WmsApi.CompleteTask(item.ID);
                }
            }
            else if (obj.Entity.HasFlag(DeviceFlags.入库))
            {
                SqlSugarHelper.Do(db =>
                {
                    //获取所有未结束的入库AGV任务
                    var agvTaks = db.Default.Queryable<WCS_AgvTaskInfo>().Where(v => v.Status < AGVTaskStatus.MissionCompleted && v.TaskType == AGVTaskType.EnterDepot).SplitTable(tabs => tabs.Take(2)).ToList().OrderBy(v => v.EditTime);

                    foreach (var agv in agvTaks)
                    {
                        //找到对应WCS任务
                        var task = db.Default.Queryable<WCS_TaskInfo>().First(v => v.AgvTaskID == agv.ID);
                        if (task == null)
                        {
                            World.Log($"AGV任务{agv.ID}未找到对应WCS任务", LogLevelEnum.Mid);
                            continue;
                        }

                        #region 开始跟据AGV状态做出处理

                        //巷道分配
                        if (agv.AgvStatus == AGVTaskStatus.RequestOrPermission1 && agv.Status != AGVTaskStatus.Complete1)
                        {
                            //向WMS获取巷道优先级
                            var res = WmsApi.GetTunnelPriorityList(task.ID);
                            var tunnelNo = res.ResData.Split(",").Select(v => "SRM" + v).ToList();

                            //开始获取堆垛机与可用站台信息
                            SRM srm = null;
                            List<Station> stations = new List<Station>();
                            if (task.Floor == 1) //一楼
                            {
                                //一楼三个入库任务
                                stations = devs.Where(v => v.Entity.Code is "1011" or "1013" or "1015").ToList();
                                //筛选出可用站台
                                stations = stations.Where(v => v.Data.VoucherNo == v.Data2.VoucherNo && v.Data3.Status.HasFlag(StatusEunm.Auto))
                                                   .Where(v => !v.Data3.Status.HasFlag(StatusEunm.PH_Status) && !v.Data3.Status.HasFlag(StatusEunm.OT_Status))
                                                   .Where(v => !v.Data3.Status.HasFlag(StatusEunm.Run)).ToList(); // 筛选出可用站台

                                //可用堆垛机
                                srm = stations.Select(v => v.Entity.Sources).SelectMany(v => v).Where(v => v.IsTunnel()) //上一个地址是巷道的
                                              .Select(v => v.Sources).SelectMany(v => v).Where(v => v.HasProtocol(typeof(ISRM520))) //筛选出堆垛机
                                              .Select(v => new SRM(v, this.World)) //转换为SRM
                                              .Where(v => !v.Data2.Status.HasFlag(SrmStatus.Alarm) && v.Data2.AutoStatus == SrmAutoStatus.Automatic) //筛选出可用堆垛机
                                              .MinBy(v => tunnelNo.IndexOf(v.Entity.Code));//按照巷道优先级排序
                            }
                            else //二楼
                            {
                            }

                            if (srm == null)
                            {
                                World.Log($"agv任务{agv.ID}无可用堆垛机", LogLevelEnum.Mid);
                                continue;
                            };
                            //筛选出堆垛机的放货站台
                            var nextPos = stations.FirstOrDefault(v => v.Entity.Sources.Where(t => t.IsTunnel()).Select(t => t.Sources).SelectMany(v => v) //获取所有巷道
                                                                                       .Where(t => t.HasProtocol(typeof(ISRM520)))//筛选出堆垛机
                                                                                       .SelectMany(v => v.Sources).Any(t => t.Code == srm.Entity.Parent.Code));
                            var agvs = db.Default.Queryable<WCS_AgvTaskInfo>().Where(v => v.Status < AGVTaskStatus.MissionCompleted && v.Position == nextPos.Entity.Code).SplitTable(v => v.Take(2)).ToList();
                            if (agvs.Any())
                            {
                                World.Log($"当前站台已有放货AGV任务执行中", LogLevelEnum.Mid);
                                continue;
                            }

                            //更新AGV任务状态
                            agv.Status = AGVTaskStatus.Complete1;
                            agv.Position = nextPos.Entity.Code;
                            db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                            //更新WCS任务状态
                            task.Device = srm.Entity.Code;
                            task.Tunnel = task.Device.GetLastDigit().ToString();
                            task.AddrNext = agv.Position;
                            db.Default.Updateable(task).ExecuteCommand();
                            task.AddWCS_TASK_DTL(db, "AGV巷道分配点", nextPos.Entity.Code, $"任务分配至堆垛机:{srm.Entity.Code}");
                            //调继续执行任务接口
                            AgvApi.ContinueTask(agv.AgvID, nextPos.Entity.Code);
                        }
                        //放货站点安全交互
                        else if (agv.AgvStatus == AGVTaskStatus.RequestOrPermission2 && agv.Status != AGVTaskStatus.Complete2)
                        {
                            var dev = devs.Find(v => v.Entity.Code == agv.Position);                            

                            if (dev.Data.VoucherNo != dev.Data2.VoucherNo)
                            {
                                World.Log($"AGV请求放货，但{obj.Entity.Code}凭证号不一致", LogLevelEnum.Mid);
                                continue;
                            }
                            if (!dev.Data3.Status.HasFlag(StatusEunm.Auto))
                            {
                                World.Log($"AGV请求放货，但{obj.Entity.Code}不在自动状态", LogLevelEnum.Mid);
                                continue;
                            }
                            if (dev.Data3.Status.HasFlag(StatusEunm.PH_Status))
                            {
                                World.Log($"AGV请求放货，但{obj.Entity.Code}光电有货", LogLevelEnum.Mid);
                                continue;
                            }
                            if (dev.Data3.Status.HasFlag(StatusEunm.OT_Status))
                            {
                                World.Log($"AGV请求放货，但{obj.Entity.Code}任务有货", LogLevelEnum.Mid);
                                continue;
                            }
                            if (dev.Data3.Status.HasFlag(StatusEunm.Run))
                            {
                                World.Log($"AGV请求放货，但{obj.Entity.Code}在运行状态", LogLevelEnum.Mid);
                                continue;
                            }
                            agv.Status = AGVTaskStatus.Complete2;
                            db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                            //调继续执行任务接口
                            AgvApi.ContinueTask(agv.AgvID, dev.Entity.Code);
                            dev.Data.TaskNumber = task.ID;
                        }
                        //完成任务
                        else if (agv.AgvStatus == AGVTaskStatus.MissionCompleted && agv.Status != AGVTaskStatus.MissionCompleted)
                        {
                            agv.Status = AGVTaskStatus.MissionCompleted;
                            db.Default.Updateable(agv).SplitTable().ExecuteCommand();
                        }

                        #endregion 开始跟据AGV状态做出处理
                    }
                });
            }
        }

        public override bool Select(Device dev)
        {
            return dev.Code is "1011" or "1012";
        }
    }
}
